@@ -21,54 +21,44 @@ Defines the :class:`Eric` class.
 
 import character
 from animatorystates import ARM_UP, BENDING_OVER, KNOCKED_OUT, RIDING_BIKE0, RIDING_BIKE1, WALK0
-from ai import Write, Jump, Hit, FireCatapult, Freeze, Catch, FireWaterPistol, DumpWaterPistol, DropStinkbomb, RideBike, FallToFloor, JumpOutOfWindow, ClimbOverSkoolGate, Kiss, ReleaseMice, SKOOL_GATE, ASSEMBLY_DIRECTION
+import ai
 import lines
 import items
-import bike
 import keys
 import sound
 
-#: Minimum delay between two non-immediate lines-givings by the same teacher.
-LINES_DELAY = 60
-
-#: The command list Mr Wacker should use to expel Eric after he's jumped out of
-#: the top-floor window.
-NOT_A_BIRD = 'ExpelEric:NotABird'
-
-#: ID of the science lab storeroom door.
-STOREROOM_DOOR = 'ScienceLabDoor'
-
-#: ID of the assembly hall.
-ASSEMBLY_HALL = 'AssemblyHall'
-
-#: Delay before Eric returns to the floor after jumping.
-JUMP_DELAY = 4
-#: Delay before Eric stands up after bending over.
-BEND_OVER_DELAY = 4
-
-#: Maximum number of mice to release per attempt.
-MAX_MICE_RELEASE = 5
-
-#: Points scored when the bike is unchained.
-BIKE_COMBO_SCORE = 1000
-#: Points scored when the storeroom key is obtained.
-STOREROOM_COMBO_SCORE = 1000
-#: Points scored when the safe key is obtained.
-SAFE_KEY_SCORE = 2000
-
-#: Number of lines a kissee does for Eric on each kiss.
-KISS_LINES = 1000
-
 class Eric(character.Character):
     """This class represents our hero.
 
     :param character_id: Unique ID.
+    :type config: dict
+    :param config: Configuration parameters from the ini file.
     :param name: Display name.
+    :param head_xy: The coordinates of Eric's head within his sprite when he's
+                    standing upright (used for collision detection).
     :param flags: Character flags.
+    :param bend_over_hand_xy: The coordinates of Eric's hand within his
+                              left-facing `BENDING_OVER` sprite (used to
+                              determine where a mouse or frog should be when
+                              caught or released).
     """
-    def __init__(self, character_id, name, flags):
-        character.Character.__init__(self, character_id, name, flags)
+    def __init__(self, character_id, config, name, head_xy, flags, bend_over_hand_xy):
+        character.Character.__init__(self, character_id, name, head_xy, flags)
         self.keyboard = None
+        self.bike_combo_score = config.get('BikeCombinationScore', 1000)
+        self.storeroom_combo_score = config.get('StoreroomCombinationScore', 1000)
+        self.safe_key_score = config.get('SafeKeyScore', 2000)
+        self.storeroom_door = config.get('StoreroomDoorId', 'ScienceLabDoor')
+        self.min_lines_delay = config.get('MinimumLinesDelay', 60)
+        self.assembly_hall = config.get('AssemblyHallId', 'AssemblyHall')
+        self.assembly_sit_direction = config.get('AssemblySitDirection', 1)
+        self.kiss_lines = config.get('KissLines', 1000)
+        self.mouse_catch_score = config.get('MouseCatchScore', 100)
+        self.max_mice_release = config.get('MaxMiceRelease', 5)
+        self.default_walk_delay = config.get('EricWalkDelay', 2)
+        self.jump_delay = config.get('JumpDelay', 4)
+        self.bend_over_delay = config.get('BendOverDelay', 4)
+        self.bend_over_hand_xy = bend_over_hand_xy
         self._reinitialise()
 
     def move(self):
@@ -84,73 +74,73 @@ class Eric(character.Character):
         self.walk_delay -= 1
         if self.walk_delay > 0:
             return 0
+        self.walk_delay = self.default_walk_delay
         self.barrier = self.skool.barrier(self) or self.cast.get_eric_stopper()
         self.floor = self.get_floor()
         self.keyboard.pump()
-        self.walk_delay = 2
         if self.controller:
-            next_controller = self.controller.command(self)
+            next_controller = self.controller.execute()
             if next_controller is self.controller:
-                self.controller = None
+                self._set_controller(None)
             elif next_controller is not None:
-                self.controller = next_controller
+                self._set_controller(next_controller)
             return self.screen.get_scroll_increment(self.x)
         if self.midstride():
             self.make_walking_sound(self.get_walk_state_index())
             return self.walk()
         if self.is_knocked_out() or self.is_sitting():
-            if self.keyboard.was_pressed(*keys.SIT_STAND):
+            if self.keyboard.was_pressed(keys.SIT_STAND):
                 if self.is_sitting_on_chair():
                     self.chair().vacate()
                 self.animatory_state = WALK0
                 self.make_sitting_sound()
-            elif self.is_sitting_on_chair() and self.keyboard.was_pressed(*keys.OPEN_DESK):
+            elif self.is_sitting_on_chair() and self.keyboard.was_pressed(keys.OPEN_DESK):
                 self.open_desk(self.skool.desk(self))
             return 0
-        if self.keyboard.was_pressed(*keys.SIT_STAND) and self.sit():
+        if self.keyboard.was_pressed(keys.SIT_STAND) and self.sit():
             self.make_sitting_sound()
             return 0
-        if self.keyboard.was_pressed(*keys.FIRE_CATAPULT) and self.can_fire_catapult():
-            self.controller = FireCatapult()
+        if self.keyboard.was_pressed(keys.FIRE_CATAPULT) and self.can_fire_catapult():
+            self._set_controller(ai.FireCatapult())
             return 0
-        if self.keyboard.was_pressed(*keys.FIRE_WATER_PISTOL) and self.can_fire_water_pistol():
-            self.controller = FireWaterPistol()
+        if self.keyboard.was_pressed(keys.FIRE_WATER_PISTOL) and self.can_fire_water_pistol():
+            self._set_controller(ai.FireWaterPistol())
             return 0
-        if self.keyboard.was_pressed(*keys.DUMP_WATER_PISTOL) and self.can_dump_water_pistol():
+        if self.keyboard.was_pressed(keys.DUMP_WATER_PISTOL) and self.can_dump_water_pistol():
             self.bend_over()
-            self.controller = DumpWaterPistol()
+            self._set_controller(ai.DumpWaterPistol())
             return 0
-        if self.keyboard.was_pressed(*keys.DROP_STINKBOMB) and self.can_drop_stinkbomb():
-            self.controller = DropStinkbomb()
+        if self.keyboard.was_pressed(keys.DROP_STINKBOMB) and self.can_drop_stinkbomb():
+            self._set_controller(ai.DropStinkbomb())
             return 0
-        if self.keyboard.was_pressed(*keys.HIT):
-            self.controller = Hit()
+        if self.keyboard.was_pressed(keys.HIT):
+            self._set_controller(ai.Hit())
             return 0
-        if self.keyboard.was_pressed(*keys.JUMP):
-            self.controller = Jump()
+        if self.keyboard.was_pressed(keys.JUMP):
+            self._set_controller(ai.Jump())
             return 0
-        if self.keyboard.was_pressed(*keys.WRITE) and self.skool.beside_blackboard(self):
+        if self.keyboard.was_pressed(keys.WRITE) and self.skool.beside_blackboard(self):
             self.raise_arm()
-            self.controller = Write()
+            self._set_controller(ai.Write())
             self.keyboard.start_writing()
             self.writing = True
             return 0
-        if self.keyboard.was_pressed(*keys.CATCH) and self.can_bend_over():
+        if self.keyboard.was_pressed(keys.CATCH) and self.can_bend_over():
             self.bend_over()
-            self.controller = Catch()
+            self._set_controller(ai.Catch())
             return 0
-        if self.keyboard.was_pressed(*keys.MOUNT_BIKE) and self.can_mount_bike():
+        if self.keyboard.was_pressed(keys.MOUNT_BIKE) and self.can_mount_bike():
             self.bike = self.cast.bike
             self.mount_bike()
-            self.controller = RideBike(self.bike)
+            self._set_controller(ai.RideBike(self.bike))
             return 0
-        if self.keyboard.was_pressed(*keys.KISS) and self.can_kiss():
+        if self.keyboard.was_pressed(keys.KISS) and self.can_kiss():
             self.previous_as = self.animatory_state
-            self.controller = Kiss()
+            self._set_controller(ai.Kiss())
             return 0
-        if self.keyboard.was_pressed(*keys.RELEASE_MICE) and self.can_release_mice():
+        if self.keyboard.was_pressed(keys.RELEASE_MICE) and self.can_release_mice():
             self.bend_over()
-            self.controller = ReleaseMice()
+            self._set_controller(ai.ReleaseMice())
             return 0
         self.staircase = self.skool.staircase(self)
         if not self.is_supported():
@@ -159,31 +149,41 @@ class Eric(character.Character):
         self.base_location = None
         old_ws, old_as, old_direction = self.get_walk_state_index(), self.animatory_state, self.direction
         plant = self.cast.plant(self)
-        if self.keyboard.pressed(*keys.LEFT):
+        if self.keyboard.pressed(keys.LEFT):
             if plant and plant.is_fully_grown():
                 barrier = self.skool.barrier(self, 1)
-                if barrier and barrier.barrier_id == SKOOL_GATE and barrier.is_shut():
-                    self.controller = ClimbOverSkoolGate()
+                if barrier and barrier.is_shut() and barrier.climb_phases:
+                    self._set_controller(ai.Flight(barrier.climb_phases))
                     return 0
             self.left()
-        elif self.keyboard.pressed(*keys.RIGHT):
+        elif self.keyboard.pressed(keys.RIGHT):
             if plant and plant.is_fully_grown():
                 window = self.skool.window(self)
-                if window and not window.is_shut():
-                    self.controller = JumpOutOfWindow(window.barrier_id)
+                if window:
+                    if not window.is_shut():
+                        self._set_controller(ai.Flight(window.climb_phases, window.not_a_bird))
                     return 0
-                elif self.barrier and self.barrier.barrier_id == SKOOL_GATE and self.barrier.is_shut():
-                    self.controller = ClimbOverSkoolGate()
+                elif self.barrier and self.barrier.is_shut() and self.barrier.climb_phases:
+                    self._set_controller(ai.Flight(self.barrier.climb_phases))
                     return 0
             self.right()
-        elif self.keyboard.pressed(*keys.UP):
+        elif self.keyboard.pressed(keys.UP):
             self.up()
-        elif self.keyboard.pressed(*keys.DOWN):
+        elif self.keyboard.pressed(keys.DOWN):
             self.down()
         if (self.animatory_state, self.direction) != (old_as, old_direction):
             self.make_walking_sound(old_ws)
         return 0
 
+    def _set_controller(self, command):
+        """Set Eric's controlling command.
+
+        :param command: The command.
+        """
+        if command:
+            command.character = self
+        self.controller = command
+
     def can_fire_catapult(self):
         """Return whether Eric can fire a catapult. This will be `True` if and
         only if :meth:`~character.Character.can_fire_catapult` (on
@@ -221,7 +221,7 @@ class Eric(character.Character):
 
         :param door: The door to check.
         """
-        if door.barrier_id == STOREROOM_DOOR and items.STOREROOM_KEY in self.inventory:
+        if door.barrier_id == self.storeroom_door and items.STOREROOM_KEY in self.inventory:
             return True
         return character.Character.can_open_door(self, door)
 
@@ -238,7 +238,7 @@ class Eric(character.Character):
         and make any nearby teacher who can see him give the lines.
         """
         self.lines_delay = max(0, self.lines_delay - 1)
-        if self.lines_delay > LINES_DELAY / 2:
+        if self.lines_delay > self.min_lines_delay / 2:
             return
         if self.lines_delay == 0:
             self.last_lines_giver = None
@@ -269,8 +269,8 @@ class Eric(character.Character):
             is_assembly = self.skool.is_assembly()
             if self.is_sitting_on_floor():
                 # Eric is sitting on the floor...
-                if room and room.room_id == ASSEMBLY_HALL:
-                    if self.direction != ASSEMBLY_DIRECTION:
+                if room and room.room_id == self.assembly_hall:
+                    if self.direction != self.assembly_sit_direction:
                         # ...facing left in the assembly hall...
                         if is_assembly:
                             # ...and it's assembly - tell him to sit facing the
@@ -284,7 +284,7 @@ class Eric(character.Character):
                     # ...somewhere other than the assembly hall - tell him to
                     # get off the floor
                     self.alert_lines_givers(lines.GET_UP, True)
-            elif self.is_standing() and home_room and home_room.contains(self) and (home_room.chairs or home_room.room_id == ASSEMBLY_HALL):
+            elif self.is_standing() and home_room and home_room.contains(self) and (home_room.chairs or home_room.room_id == self.assembly_hall):
                 # Eric is standing up either in a classroom after the lesson
                 # has started, or in the assembly hall after assembly has
                 # started - tell him to sit down
@@ -320,7 +320,7 @@ class Eric(character.Character):
         if self.writing:
             self.writing = False
             self.keyboard.finish_writing()
-            self.controller = None
+            self._set_controller(None)
         self.play_sound(sound.KNOCKED_OUT, sound.ASYNC)
 
     def alert_lines_givers(self, message_id, reset_delay=False):
@@ -331,6 +331,8 @@ class Eric(character.Character):
         :param reset_delay: Whether to reset the lines-giving delay.
         """
         nearby_lines_givers = self.cast.get_nearby_lines_givers(self)
+        if reset_delay and self.last_lines_giver in nearby_lines_givers:
+            nearby_lines_givers.remove(self.last_lines_giver)
         if nearby_lines_givers:
             lines_giver = nearby_lines_givers[0]
             if message_id == lines.GET_ALONG:
@@ -341,9 +343,7 @@ class Eric(character.Character):
                 elif self.skool.in_playground(self):
                     message_id = lines.BACK_TO_SKOOL
             if reset_delay:
-                if lines_giver is self.last_lines_giver:
-                    return
-                self.lines_delay = LINES_DELAY
+                self.lines_delay = self.min_lines_delay
             lines_giver.give_lines(self.character_id, message_id)
             self.last_lines_giver = lines_giver
 
@@ -389,13 +389,13 @@ class Eric(character.Character):
             self.inventory.remove(pistol)
             self.print_inventory()
             self.skool.hide_in_desk(items.WATER_PISTOL)
-        self.walk_delay = BEND_OVER_DELAY
+        self.walk_delay = self.bend_over_delay
 
     def fire_water_pistol(self):
         """Make Eric fire the water pistol. Also alert any lines-givers in the
         vicinity, and play a sound effect.
         """
-        liquid = character.SHERRY if items.SHERRY_PISTOL in self.inventory else character.WATER
+        liquid = self.cast.sherry_id if items.SHERRY_PISTOL in self.inventory else self.cast.water_id
         character.Character.fire_water_pistol(self, liquid)
         self.alert_lines_givers(lines.NO_WATERPISTOLS)
         self.play_sound(sound.WATER_PISTOL, sound.ASYNC)
@@ -463,7 +463,7 @@ class Eric(character.Character):
             self.base_location = (self.x, self.y)
         self.previous_as = self.animatory_state
         self.y -= 1
-        self.walk_delay = JUMP_DELAY
+        self.walk_delay = self.jump_delay
         self.animatory_state = ARM_UP
         self.alert_lines_givers(lines.NO_JUMPING)
 
@@ -498,14 +498,22 @@ class Eric(character.Character):
         self.walk_delay = 1
         self.animatory_state = BENDING_OVER
 
+    def _get_bend_over_hand_coords(self):
+        """Return the coordinates of Eric's hand when he's bending over."""
+        if not self.bend_over_hand_xy:
+            return
+        hand_x = self.x + (self.bend_over_hand_xy[0] if self.direction < 0 else self.width - self.bend_over_hand_xy[0] - 1)
+        hand_y = self.y + self.bend_over_hand_xy[1]
+        return hand_x, hand_y
+
     def catch_animal(self):
         """Make Eric catch any animal that is close at hand. If Eric catches
         something, a sound effect is played.
         """
-        animal = self.cast.get_animal(self)
+        animal = self.cast.get_animal(*self._get_bend_over_hand_coords())
         if animal:
             if animal.is_mouse():
-                self.skool.add_to_score(10)
+                self.skool.add_to_score(self.mouse_catch_score)
                 self.play_sound(sound.MOUSE)
                 self.mice += 1
                 self.print_mouse_inventory()
@@ -515,7 +523,7 @@ class Eric(character.Character):
                 self.play_sound(sound.FROG)
                 self.inventory.add(items.FROG)
                 self.print_inventory()
-        self.walk_delay = BEND_OVER_DELAY
+        self.walk_delay = self.bend_over_delay
 
     def print_mouse_inventory(self):
         """Print Eric's mouse inventory."""
@@ -531,11 +539,11 @@ class Eric(character.Character):
     def release_mice(self):
         """Make Eric release some mice."""
         if self.mice > 0:
-            num_mice = min(self.mice, MAX_MICE_RELEASE)
+            num_mice = min(self.mice, self.max_mice_release)
             self.mice -= num_mice
             self.print_mouse_inventory()
-            self.cast.release_mice(num_mice, self.x + self.direction, self.y)
-        self.walk_delay = BEND_OVER_DELAY
+            self.cast.release_mice(num_mice, *self._get_bend_over_hand_coords())
+        self.walk_delay = self.bend_over_delay
 
     def print_inventory(self):
         """Print Eric's inventory."""
@@ -548,13 +556,17 @@ class Eric(character.Character):
 
     def get_location(self):
         """Return the on-floor location that is closest to Eric."""
+        x, y = self.x, self.y
         if self.hide_coords:
             # If Eric is hiding (e.g. while kissing), temporarily restore his
             # pre-hide coordinates for the location check
             self.x, self.y = self.hide_coords
+        elif self.base_location:
+            # If Eric is jumping, temporarily restore his pre-jump coordinates
+            # for the location check
+            self.x, self.y = self.base_location
         location = character.Character.get_location(self)
-        if self.hide_coords:
-            self.hide()
+        self.x, self.y = x, y
         return location
 
     def get_location_above_hand(self):
@@ -585,7 +597,7 @@ class Eric(character.Character):
                 self.keyboard.finish_writing()
                 return
             char = first_event.unicode
-            if self.skool.font.has_char(char):
+            if self.screen.has_font_char(char):
                 self.raise_arm()
                 self.skool.write_on_board(self, blackboard, char)
         else:
@@ -600,11 +612,11 @@ class Eric(character.Character):
         room and play a sound effect.
         """
         if not self.cast.is_bike_visible() and self.skool.got_bike_combination(blackboard):
-            self.skool.add_to_score(BIKE_COMBO_SCORE // 10)
+            self.skool.add_to_score(self.bike_combo_score)
             self.skool.unchain_bike()
             self.play_sound(sound.BIKE)
         elif items.STOREROOM_KEY not in self.inventory and self.skool.got_storeroom_combination(blackboard):
-            self.skool.add_to_score(STOREROOM_COMBO_SCORE // 10)
+            self.skool.add_to_score(self.storeroom_combo_score)
             self.inventory.add(items.STOREROOM_KEY)
             self.print_inventory()
             self.play_sound(sound.STOREROOM_KEY)
@@ -618,19 +630,20 @@ class Eric(character.Character):
 
     def freeze(self):
         """Attempt to freeze Eric. The attempt will fail if Eric is writing on
-        a blackboard.
+        a blackboard, or is already frozen.
 
         :return: `True` if Eric was frozen, `False` otherwise.
         """
-        if not self.writing:
-            self.controller = Freeze()
-            self.frozen = True
-        return self.frozen
+        if self.writing or self.frozen:
+            return False
+        self._set_controller(ai.Freeze())
+        self.frozen = True
+        return True
 
     def unfreeze(self):
         """Unfreeze Eric."""
         if self.frozen:
-            self.controller = None
+            self._set_controller(None)
             self.frozen = False
             self.understood = False
 
@@ -638,25 +651,28 @@ class Eric(character.Character):
         """Check whether Eric indicated understanding of the message just
         delivered to him.
         """
-        if self.keyboard.was_pressed(*keys.UNDERSTOOD):
+        if self.keyboard.was_pressed(keys.UNDERSTOOD):
             self.understood = True
 
     def understood_message(self):
         """Return whether Eric understood the message just delivered to him."""
         return self.understood
 
-    def paralyse(self):
+    def paralyse(self, command_list_id):
         """Signal that Eric is paralysed (as after falling from the top-floor
         window). Mr Wacker will be alerted.
+
+        :param command_list_id: The ID of the command list Mr Wacker should
+                                switch to.
         """
-        self.cast.expel_eric(NOT_A_BIRD)
+        self.skool.expel_eric(command_list_id)
 
     def take_safe_key(self):
         """Make Eric collect the safe key. A celebratory sound effect will be
         played.
         """
         if items.SAFE_KEY not in self.inventory:
-            self.skool.add_to_score(SAFE_KEY_SCORE // 10)
+            self.skool.add_to_score(self.safe_key_score)
             self.play_sound(sound.SAFE_KEY, sound.SUSPEND)
             self.inventory.add(items.SAFE_KEY)
             self.print_inventory()
@@ -673,7 +689,7 @@ class Eric(character.Character):
 
     def kiss(self):
         """Decrease Eric's lines total and play the kissing sound effect."""
-        self.skool.add_lines(-KISS_LINES // 10)
+        self.skool.add_lines(-self.kiss_lines)
         self.play_sound(sound.KISS)
 
     def hide(self):
@@ -698,7 +714,7 @@ class Eric(character.Character):
 
     def fall_off_plant(self):
         """Make Eric fall off a plant that has just died."""
-        self.controller = FallToFloor()
+        self._set_controller(ai.FallToFloor())
 
     def fly(self, x_inc, y_inc):
         """Move Eric to the next spot in his flight out of a window or over the
@@ -731,8 +747,8 @@ class Eric(character.Character):
         self.writing = False
         self.frozen = False
         self.understood = False
-        self.controller = None
-        self.lines_delay = LINES_DELAY
+        self._set_controller(None)
+        self.lines_delay = self.min_lines_delay
         self.last_lines_giver = None
         self.inventory = set()
         self.mice = 0
@@ -768,29 +784,24 @@ class Eric(character.Character):
         bike key pressed is stored and acted upon later by another bike-related
         method.
         """
-        self.walk_delay = bike.SPEED
         self.bike_key = None
         for key in keys.LEFT + keys.RIGHT + keys.UP + keys.DOWN + keys.JUMP:
             if self.keyboard.was_pressed(key):
                 self.bike_key = key
                 break
 
-    def rode_into_barrier(self, bike, barrier_id=None):
-        """Return whether Eric rode the bike into a wall or door.
+    def hit_barrier(self, bike):
+        """Return the wall or door that Eric rode the bike into (if any).
 
         :param bike: The bike Eric is riding.
-        :param barrier_id: The ID of the wall or door to check; if `None`, all
-                           doors and walls are checked.
+        :return: The barrier that was hit, or `None` if none was hit.
         """
         if bike.is_visible():
-            return bike.hit_barrier(barrier_id)
+            return bike.hit_barrier()
         # Check from a distance of 1 if travelling leftwards so that the bike
         # does not appear too close to the barrier if it hits it
         distance = 0 if self.direction > 0 else 1
-        barrier = self.skool.barrier(self, distance)
-        if barrier_id:
-            return barrier and barrier.barrier_id == barrier_id
-        return barrier is not None
+        return self.skool.barrier(self, distance)
 
     def pedalled(self):
         """Return whether Eric pedalled the bike since the last time he was
@@ -900,7 +911,7 @@ class Eric(character.Character):
     #//////////////////////////////////////////////////////////////////////////
     # Sound effects
     #//////////////////////////////////////////////////////////////////////////
-    def play_sound(self, sound_id, mode=sound.BLOCK):
+    def play_sound(self, sound_id, mode=sound.SUSPEND):
         """Play a sound effect.
 
         :param sound_id: The ID of the sound effect.
