@@ -17,6 +17,7 @@
 # Pyskool. If not, see <http://www.gnu.org/licenses/>.
 
 import btsini
+import skoolbuilder
 from skooltaps import *
 from skoolids import *
 
@@ -1045,7 +1046,7 @@ class BTSTapMaker:
         lesson.add_entry(HEROINE, CL_UPPER_ROOM_GIRL)
         lessons.append(lesson)
 
-        lesson = Lesson(LSN_DINNER_WACKER, '*%s' % WACKER, DINNER_HALL)
+        lesson = Lesson(LSN_DINNER_WACKER, '%s%s' % (skoolbuilder.HIDE_TEACHER_MARKER, WACKER), DINNER_HALL)
         lesson.add_entry(GIRL01, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL02, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL03, CL_DINNER_HALL_GIRL)
@@ -1075,7 +1076,7 @@ class BTSTapMaker:
         lesson.add_entry(HEROINE, CL_DINNER_HALL_GIRL)
         lessons.append(lesson)
 
-        lesson = Lesson(LSN_DINNER_WITHIT, '*%s' % WITHIT, DINNER_HALL)
+        lesson = Lesson(LSN_DINNER_WITHIT, '%s%s' % (skoolbuilder.HIDE_TEACHER_MARKER, WITHIT), DINNER_HALL)
         lesson.add_entry(GIRL01, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL02, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL03, CL_DINNER_HALL_GIRL)
@@ -1105,7 +1106,7 @@ class BTSTapMaker:
         lesson.add_entry(HEROINE, CL_DINNER_HALL_GIRL)
         lessons.append(lesson)
 
-        lesson = Lesson(LSN_ASSEMBLY, '*%s' % WITHIT, ASSEMBLY_HALL)
+        lesson = Lesson(LSN_ASSEMBLY, '%s%s' % (skoolbuilder.HIDE_TEACHER_MARKER, WITHIT), ASSEMBLY_HALL)
         lesson.add_entry(GIRL01, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL02, CL_DINNER_HALL_GIRL)
         lesson.add_entry(GIRL03, CL_DINNER_HALL_GIRL)
